@@ -1,7 +1,7 @@
 //! Block size parameters
 
 use crate::error::Error;
-use std::convert::{TryFrom, TryInto};
+use core::convert::{TryFrom, TryInto};
 use tendermint_proto::Protobuf;
 use {
     crate::serializers,
